from arepo.base import Base

from sqlalchemy import Column, Integer, String, Boolean, ForeignKey, ForeignKeyConstraint
from sqlalchemy.orm import relationship


class RepositoryModel(Base):
    __tablename__ = "repository"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    owner = Column('owner', String, nullable=False)
    available = Column('available', Boolean, nullable=True)
    description = Column('description', String, nullable=True)
    language = Column('language', String, nullable=True)
    size = Column('size', Integer, nullable=True)
    watchers = Column('watchers', Integer, nullable=True)
    forks = Column('forks', Integer, nullable=True)
    stargazers = Column('stargazers', Integer, nullable=True)
    commits_count = Column('commits_count', Integer, nullable=True)
    commits = relationship("CommitModel", backref="repository")

    def __repr__(self):
        return f"<Repository {self.owner}/{self.name}>"


class RepositoryAssociationModel(Base):
    __tablename__ = "repository_association"
    __table_args__ = (
        ForeignKeyConstraint(('repository_id',), ['repository.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
    )

    repository_id = Column(String, ForeignKey('repository.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)



from arepo.base import Base

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, ForeignKeyConstraint


class TagModel(Base):
    __tablename__ = "tag"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    references = relationship("ReferenceModel", secondary="reference_tag", backref='tags')


class ReferenceModel(Base):
    __tablename__ = "reference"

    id = Column('id', String, primary_key=True)
    url = Column('url', String, nullable=False)


class ReferenceTagModel(Base):
    __tablename__ = 'reference_tag'
    __table_args__ = (
        ForeignKeyConstraint(('reference_id',), ['reference.id']),
        ForeignKeyConstraint(('tag_id',), ['tag.id']),
    )

    reference_id = Column(String, ForeignKey('reference.id'), primary_key=True)
    tag_id = Column(Integer, ForeignKey('tag.id'), primary_key=True)


class ReferenceAssociationModel(Base):
    __tablename__ = "reference_association"
    __table_args__ = (
        ForeignKeyConstraint(('reference_id',), ['reference.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
    )

    reference_id = Column(String, ForeignKey('reference.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)

    # Define the relationships
    vulnerability = relationship("VulnerabilityModel", backref="reference_vulnerabilities")
    reference = relationship("ReferenceModel", backref="reference_vulnerabilities")
    source = relationship("SourceModel", backref="reference_vulnerabilities")



from arepo.base import Base

from sqlalchemy.orm import relationship
from sqlalchemy import Column, String, ForeignKey, Integer


class ProductModel(Base):
    __tablename__ = "product"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    part = Column('part', String, nullable=False)
    vendor_id = Column(String, ForeignKey('vendor.id'), nullable=False)
    # TODO: add more fields
    vendor = relationship("VendorModel", back_populates="products")
    cpes = relationship("CPEModel", back_populates="product")



from arepo.base import Base

from sqlalchemy import Column, String


class SourceModel(Base):
    __tablename__ = "source"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    email = Column('email', String, nullable=False)



from arepo.base import Base
from sqlalchemy.orm import relationship
from sqlalchemy import Column, String


class VendorModel(Base):
    __tablename__ = "vendor"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    # TODO: add more fields
    products = relationship("ProductModel", back_populates="vendor")
